@@ -21,34 +21,37 @@
 #include <gtk/gtk.h>
 #include "press_data.h"
 
-
 /*
  * Constants
  */
-#define SURFACE_WIDTH 920
-#define SURFACE_HEIGHT 650
-#define GAME_EVENTS (GDK_EXPOSURE_MASK              |\
-							GDK_BUTTON_PRESS_MASK          |\
-							GDK_POINTER_MOTION_MASK        |\
-							GDK_BUTTON_RELEASE_MASK)
-	  
-	  
+#define GAMESDIR "sol-games/"
+#define GAME_EVENTS (GDK_EXPOSURE_MASK        |\
+		     GDK_BUTTON_PRESS_MASK    |\
+		     GDK_POINTER_MOTION_MASK  |\
+		     GDK_BUTTON_RELEASE_MASK)
+/*
+ * Global variables
+ */
+extern GtkWidget        *app;
+extern GtkWidget        *playing_area;
+extern GdkGC            *draw_gc;
+extern GdkPixmap        *surface;
+extern GdkPixmap        *moving_card_pixmap;
+extern guint            score;
+extern guint            game_time;
+extern guint            timeout;
+extern guint            seed;
+extern guint            n_games;
+extern struct dirent    **game_dents;
+extern gchar            *game_file;
+extern gchar            *game_name;
+extern press_data_type* press_data; 
 
-#ifndef SOL_C
-extern GtkWidget* playing_area;
-extern GtkWidget* app;
-extern GdkPixmap *snapshot;
-extern GdkPixmap *blank_surface;
-extern GdkPixmap *surface;
-extern GdkPixmap *moving_card_pixmap;
-gint score;
-extern GtkWidget *scorew;
-extern gint seed;
-extern GtkWidget *label;
-extern GString* game;
-#endif
+gchar* game_file_to_name(const gchar* file);
 void set_score();
+void timer_start();
+void timer_stop();
 void make_title();
-void eval_installed_file(char *file);
+void eval_installed_file(gchar *file);
 
 #endif
