@@ -27,6 +27,7 @@
 
 #include "gnothello.h"
 #include "othello.h"
+#include "network.h"
 
 GtkWidget *window;
 GtkWidget *drawing_area;
@@ -77,8 +78,11 @@ int session_position = 0;
 
 static struct argp_option options[] =
 {
-	{NULL, 'x', N_("X"), OPTION_HIDDEN, NULL, 1},
-	{NULL, 'y', N_("Y"), OPTION_HIDDEN, NULL, 1},
+	{NULL, 'x', N_("X"),   OPTION_HIDDEN, NULL, 1},
+	{NULL, 'y', N_("Y"),   OPTION_HIDDEN, NULL, 1},
+#ifdef HAVE_ORBIT
+	{"ior",'i', N_("IOR"), 0,             N_("IOR of remote Gnothello server"), 1 },
+#endif
 	{NULL, 0, NULL, 0, NULL, 0}
 };
 
@@ -89,24 +93,26 @@ static struct argp parser =
 };
 
 GnomeUIInfo file_menu[] = {
-	GNOMEUIINFO_ITEM_STOCK(N_("New Game"), NULL, new_game_cb, "Menu_New"),
-	GNOMEUIINFO_ITEM_STOCK(N_("Quit"), NULL, quit_game_cb, "Menu_Quit"),
+	{ GNOME_APP_UI_ITEM, N_("_New"), NULL, new_game_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 'n', GDK_CONTROL_MASK },
+	{ GNOME_APP_UI_ITEM, N_("_Quit"), NULL, quit_game_cb, NULL, NULL,
+	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXIT, 'q', GDK_CONTROL_MASK },
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo black_level_radio_list[] = {
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Disabled"), NULL, black_level_cb, "0", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level One"), NULL, black_level_cb, "1", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level Two"), NULL, black_level_cb, "2", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level Three"), NULL, black_level_cb, "3", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("_Disabled"), NULL, black_level_cb, "0", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _One"), NULL, black_level_cb, "1", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _Two"), NULL, black_level_cb, "2", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _Three"), NULL, black_level_cb, "3", NULL),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo white_level_radio_list[] = {
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Disabled"), NULL, white_level_cb, "0", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level One"), NULL, white_level_cb, "1", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level Two"), NULL, white_level_cb, "2", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Level Three"), NULL, white_level_cb, "3", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("_Disabled"), NULL, white_level_cb, "0", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _One"), NULL, white_level_cb, "1", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _Two"), NULL, white_level_cb, "2", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("Level _Three"), NULL, white_level_cb, "3", NULL),
 	GNOMEUIINFO_END
 };
 
@@ -121,38 +127,38 @@ GnomeUIInfo white_level_menu[] = {
 };
 
 GnomeUIInfo comp_menu[] = {
-	GNOMEUIINFO_SUBTREE(N_("Black"), black_level_menu),
-	GNOMEUIINFO_SUBTREE(N_("White"), white_level_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Black"), black_level_menu),
+	GNOMEUIINFO_SUBTREE(N_("_White"), white_level_menu),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_TOGGLEITEM(N_("Quick Moves"), NULL, quick_moves_cb, NULL),
+	GNOMEUIINFO_TOGGLEITEM(N_("_Quick Moves"), NULL, quick_moves_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo anim_radio_list[] = {
-	GNOMEUIINFO_RADIOITEM_DATA(N_("No Animation"), NULL, anim_cb, "0", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Some Animation"), NULL, anim_cb, "1", NULL),
-	GNOMEUIINFO_RADIOITEM_DATA(N_("Full Animation"), NULL, anim_cb, "2", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("_No Animation"), NULL, anim_cb, "0", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("_Some Animation"), NULL, anim_cb, "1", NULL),
+	GNOMEUIINFO_ITEM_DATA(N_("_Full Animation"), NULL, anim_cb, "2", NULL),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo anim_menu[] = {
 	GNOMEUIINFO_RADIOLIST(anim_radio_list),
 	GNOMEUIINFO_SEPARATOR,
-	GNOMEUIINFO_TOGGLEITEM(N_("Stagger Flips"), NULL, anim_stagger_cb, NULL),
+	GNOMEUIINFO_TOGGLEITEM(N_("_Stagger Flips"), NULL, anim_stagger_cb, NULL),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo help_menu[] = {
-	GNOMEUIINFO_ITEM_STOCK(N_("About..."), NULL, about_cb, "Menu_About"),
+	GNOMEUIINFO_ITEM_STOCK(N_("_About..."), NULL, about_cb, "Menu_About"),
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo mainmenu[] = {
-	GNOMEUIINFO_SUBTREE(N_("Game"), file_menu),
-	GNOMEUIINFO_SUBTREE(N_("Computer"), comp_menu),
-	GNOMEUIINFO_SUBTREE(N_("Animation"), anim_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Game"), file_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Computer"), comp_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Animation"), anim_menu),
 	GNOMEUIINFO_JUSTIFY_RIGHT,
-	GNOMEUIINFO_SUBTREE(N_("Help"), help_menu),
+	GNOMEUIINFO_SUBTREE(N_("_Help"), help_menu),
 	GNOMEUIINFO_END
 };
 
@@ -167,6 +173,11 @@ static error_t parse_args(int key, char *arg, struct argp_state *state)
 			session_flag |= 2;
 			session_ypos = atoi(arg);
 			break;
+#ifdef HAVE_ORBIT
+	        case 'i':
+			ior = arg;
+			break;
+#endif
 		case ARGP_KEY_SUCCESS:
 			if(session_flag == 3) session_position = 1;
 			break;
@@ -199,6 +210,7 @@ void quit_game_cb(GtkWidget *widget, gpointer data)
 
 void new_game_cb(GtkWidget *widget, gpointer data)
 {
+	network_new ();
 	init_new_game();
 }
 
@@ -329,6 +341,9 @@ gint button_press_event(GtkWidget *widget, GdkEventButton *event)
 {
 	guint x, y;
 
+	if (!network_allow ())
+		return (TRUE);
+	
 	if(whose_turn == WHITE_TURN)
 		if(gnome_config_get_int("/gnothello/Preferences/whitelevel=0"))
 			return(TRUE);
@@ -341,7 +356,7 @@ gint button_press_event(GtkWidget *widget, GdkEventButton *event)
 		x = event->x / TILEWIDTH;
 		y = event->y / TILEHEIGHT;
 		if(is_valid_move(x, y, whose_turn))
-			move(x, y, whose_turn);
+			game_move(x, y, whose_turn);
 	}
 
 	return(TRUE);
@@ -370,7 +385,7 @@ void load_pixmaps()
 	g_free(tmp);
 
 	if(!g_file_exists(fname)) {
-		g_print(N_("Could not fine \'%s\' pixmap file for Gnothello\n"), fname);
+		g_print(N_("Could not find \'%s\' pixmap file for Gnothello\n"), fname);
 		exit(1);
 	}
 
@@ -624,6 +639,7 @@ static int save_state(GnomeClient *client, gint phase, GnomeRestartStyle save_st
 int main(int argc, char **argv)
 {
 	GnomeClient *client;
+	CORBA_def(CORBA_Environment ev;)
 	struct timeval tv;
 
 	argp_program_version = GNOTHELLO_VERSION;
@@ -642,9 +658,13 @@ int main(int argc, char **argv)
 	gtk_signal_connect(GTK_OBJECT(client), "save_yourself", GTK_SIGNAL_FUNC(save_state), argv[0]);
 	gtk_signal_connect(GTK_OBJECT(client), "die", GTK_SIGNAL_FUNC(quit_game_cb), argv[0]);
 
+#ifdef HAVE_ORBIT
+	CORBA_exception_init (&ev);
+	orb = gnome_CORBA_init ("gnothello", &parser, &argc, argv, 0, NULL);
+#else
 	gnome_init("gnothello", &parser, argc, argv, 0, NULL);
-	gdk_imlib_init();
-
+#endif
+	
 	create_window();
 	create_menus();
 	create_drawing_area();
@@ -665,6 +685,7 @@ int main(int argc, char **argv)
 
 	gdk_window_clear_area(drawing_area->window, 0, 0, BOARDWIDTH, BOARDHEIGHT);
 
+	network_init();
 	gtk_main();
 
 	gtk_object_unref(GTK_OBJECT(client));
