@@ -26,26 +26,23 @@ int global_wait = 0 ;
 #endif
 
 struct _dep_tree {
-  int turn_dep[4] ;	// Turning dependancies all must be clear 
-  int place_dep[4] ;	// Placing dependancies all must be full
-  int lhs_dep[2] ;	// Sideways dependancies on this level
-  int rhs_dep[2] ;	// N.B. Repeats are allowed.
-  int free ;		// Can we put a tile in it ?
-  int filled ;		// Does it have a tile in it ?
+  int turn_dep[4] ;	/* Turning dependancies all must be clear */
+  int place_dep[4] ;	/* Placing dependancies all must be full */
+  int lhs_dep[2] ;	/* Sideways dependancies on this level */
+  int rhs_dep[2] ;	/* N.B. Repeats are allowed. */
+  int free ;		/* Can we put a tile in it ? */
+  int filled ;		/* Does it have a tile in it ? */
 };
 struct _dep_tree dep_tree[MAX_TILES] ;
      
-struct _sensible_types {
-  int def[2] ;
-  int placed ;
-} ;
-struct _sensible_types sensible_types[(MAX_TILES/2)] ;
-
 /* N.B. The +1.0 _is_ intended ! */
 #define random_UDL ((int)( (((float)MAX_TILES) * rand())/(1.0 + RAND_MAX)))
 #define random_UDL_half ((int)( (((float)(MAX_TILES/2)) * rand())/(1.0 + RAND_MAX)))
 #define RANDOM_LIMIT (MAX_TILES*MAX_TILES)
 
+/* The number of generation failures */
+static int fails = 0 ;
+
 #ifdef PLACE_DEBUG
 void dump_deps (int lp)
 {
@@ -61,11 +58,67 @@ void dump_deps (int lp)
 
 void dump_tile (int t)
 {
-  printf ("Tile %d at (%d, %d), layer %d free %d filled %d\n", t, default_pos[t].x, default_pos[t].y, default_pos[t].layer, dep_tree[t].free, dep_tree[t].filled) ;
+  printf ("Tile %d at (%d, %d), layer %d free %d filled %d\n", t, pos[t].x, pos[t].y, pos[t].layer, dep_tree[t].free, dep_tree[t].filled) ;
+}
+
+void keyPress()
+{
+    if (global_wait)
+    {
+	int done = 0 ;
+	while (!done)
+	{
+	    int zlp ;
+	    char b ;
+
+	    done = 1 ;
+	    clear_window() ;	
+	    for (zlp=MAX_TILES-1;zlp>-1;zlp--)
+		redraw_tile(zlp) ;
+	    for (zlp=0;zlp<MAX_TILES;zlp++) {
+		tiles[zlp].selected = 0 ;
+		if (tiles[zlp].visible == 17)
+		    tiles[zlp].visible = 0 ;
+	    }
+	    printf ("Drawn\n") ;
+	    while (fread(&b, 1, 1, stdin)==0) ;
+	    if (b=='c')
+	    {
+		printf ("All :\n") ;
+		for (zlp=0;zlp<MAX_TILES;zlp++)
+		    validate_tile(zlp) ;
+		done = 0 ;
+		b='s' ;
+	    }
+	    if (b=='s')
+	    {
+		printf ("Free :\n") ;
+		for (zlp=0;zlp<MAX_TILES;zlp++) {
+		    tiles[zlp].selected = dep_tree[zlp].free ;
+		    if (tiles[zlp].selected)
+			tiles[zlp].visible = 17 ;
+		}
+		done = 0 ;
+	    }
+	    if (b=='q')
+		global_wait = 0 ;
+	    if (b=='a')
+	    {
+		printf ("All :\n") ;
+		for (zlp=0;zlp<MAX_TILES;zlp++)
+		    if (!tiles[zlp].visible)
+			tiles[zlp].visible = 17 ;
+		done = 0 ;
+	    }
+	    else if (b=='r') done = 0 ;
+	    while (b!='\n' && fread(&b, 1, 1, stdin)==0) ;
+	    printf ("Read '%c'\n", b) ;
+	}
+    }
 }
 #endif
 
-/* Takes the last tile places, and avoids its dependancies */
+/* Takes the last tile places, and avoids its dependancies. */
 int random_free (int lt)
 {
   int t, lp, catch ;
@@ -111,80 +164,107 @@ int random_on_layer (int l)
     {
       t = random_UDL ;
       if (cnt++>RANDOM_LIMIT)
-	longjmp (unsolvable,1) ;
+	longjmp (unsolvable,2) ;
     }
-  while (default_pos[t].layer!=l) ;
+  while (pos[t].layer!=l) ;
 
   assert (!dep_tree[t].filled) ;
   return t ;
 }
 
+/* x,y specify the position of a _quarter_ tile, and layer its layer.
+   This returns the offset of the first tile to intersect the x,y _quarter_
+   it returns -1 if there is no tile at that position.
+   N.B. Each tile occupies 2x2 positions.
+ */
+int tile_at (int layer, int x, int y)
+{
+  int lp ;
+
+  for (lp=0;lp<MAX_TILES;lp++)
+    if ((pos[lp].layer == layer) &&
+	(pos[lp].x==x || pos[lp].x==x+1) &&
+	(pos[lp].y==y || pos[lp].y==y+1))
+      return lp ;
+  return -1 ;
+}
+
 /* This determines whether it is OK to mark this tile as free */
 int ok_free_validate_line (int t)
 {
-  int i, w ;
+  int i, li, w ;
   int in, out ;
-  int next[MAX_TILES] ;
+  static int nextl[MAX_TILES] ;
+  static int nextr[MAX_TILES] ;
 
   /* Go left */
   in = 0 ;
   out = 0 ;
-  next[in++] = t ;
+  nextl[in++] = t ;
   do
     {
-      w = next[out++] ;
+      w = nextl[out++] ;
       if (dep_tree[w].free || dep_tree[w].filled)
 	return 0 ;
       if ((i=dep_tree[w].lhs_dep[0])!=-1)
-	next[in++] = i ;
-      if ((i=dep_tree[w].lhs_dep[1])!=-1)
-	next[in++] = i ;
+	nextl[in++] = i ;
+      li = i ;
+      if ((i=dep_tree[w].lhs_dep[1])!=-1 && li != i)
+	nextl[in++] = i ;
     }
   while (in>out) ;
 
-#ifdef PLACE_DEBUG
-  printf ("%d's lhs line : ", t) ;
-  for (lp=0;lp<in;lp++)
-    printf ("%d ", next[lp]) ;
-  printf ("\n") ;
-#endif
-
   /* Go right */
   in = 0 ;
   out = 0 ;
-  next[in++] = t ;
+  nextr[in++] = t ;
   do
     {
-      w = next[out++] ;
+      w = nextr[out++] ;
       if (dep_tree[w].free || dep_tree[w].filled)
 	return 0 ;
       if ((i=dep_tree[w].rhs_dep[0])!=-1)
-	next[in++] = i ;
-      if ((i=dep_tree[w].rhs_dep[1])!=-1)
-	next[in++] = i ;
+	nextr[in++] = i ;
+      li = i ;
+      if ((i=dep_tree[w].rhs_dep[1])!=-1 && li != i)
+	nextr[in++] = i ;
     }
   while (in>out) ;
 
 #ifdef PLACE_DEBUG
+  printf ("FREE on this row !\n") ;
+
   printf ("%d's rhs line : ", t) ;
-  for (lp=0;lp<in;lp++)
-    printf ("%d ", next[lp]) ;
+  for (i=0;i<in;i++)
+    printf ("%d ", nextr[i]) ;
   printf ("\n") ;
+
+  printf ("%d's lhs line : ", t) ;
+  for (i=0;i<in;i++)
+    printf ("%d ", nextl[i]) ;
+  printf ("\n") ;
+
 #endif
   
   return 1 ;
 }
 
-/* This simple examines whether the tile should be free or not.
-   It does this by examining its dependancies to see if they are filled */
+/* This simply examines whether the tile should be free or not.
+   It does this by examining its dependancies to see if they are
+   filled */
 void validate_tile (int t)
 {
-  int lfilled, rfilled, lp, free, i, valid;
+  int lfilled, rfilled, lp, free, i, valid ;
 
   if (t==-1||dep_tree[t].filled)
-    return ;	// No point.
+    return ;	/* No point. */
 
 #ifdef PLACE_DEBUG
+  if (global_wait==0)
+  {
+      dep_tree[t].free = 1 ;
+      return ;
+  }
   printf ("Validating : %d\n", t) ;
   dump_deps(t) ;
 #endif
@@ -197,9 +277,9 @@ void validate_tile (int t)
     if ((i = dep_tree[t].place_dep[lp]) != -1)
       if (!dep_tree[i].filled) return ;
 
-  if (ok_free_validate_line(t))	// First in this layer on this line
+  if (ok_free_validate_line(t))	/* First in this layer on this line */
     {
-      assert (default_pos[t].layer!=0) ;
+      assert (pos[t].layer!=0) ; /* Intended */
       dep_tree[t].free = 1 ;
       return ;
     }
@@ -207,7 +287,7 @@ void validate_tile (int t)
   /* LHS */
   if ((dep_tree[t].lhs_dep[0] == -1) &&
       (dep_tree[t].lhs_dep[1] == -1))
-    lfilled = 0 ;
+    lfilled = 0 ;  /* ie. so being on the edge != can put a tile there */
   else
     {
       lfilled = 1 ;      
@@ -236,7 +316,7 @@ void validate_tile (int t)
 }
 
 /* Place tile in map at position f, with pic & type from t */
-void place_tile (int f, int t)
+void place_tile (int f, int t, int idx)
 {
 #ifdef PLACE_DEBUG
   printf ("Placing at\n") ;
@@ -244,21 +324,23 @@ void place_tile (int f, int t)
 #endif
   tiles[f].visible = 1;
   tiles[f].selected = 0;
-  tiles[f].x = default_pos[f].x * (HALF_WIDTH-0) + 30 + (5 * default_pos[f].layer);
-  tiles[f].y = default_pos[f].y * (HALF_HEIGHT-0) + 25 - (4 * default_pos[f].layer);
-  tiles[f].layer = default_pos[f].layer;
-  //  tiles[f].sequence = 0 ;
+  tiles[f].x = pos[f].x * (HALF_WIDTH-0) + 120 + (5 * pos[f].layer);
+  tiles[f].y = pos[f].y * (HALF_HEIGHT-0) + 25 - (4 * pos[f].layer);
+  tiles[f].layer = pos[f].layer;
+  /*  if (tiles[f].layer>0)
+      tiles[f].visible = 0 ;
+      tiles[f].sequence = 0 ; */
 
-  tiles[f].type = default_types[t].type; 
-  tiles[f].image = default_types[t].image;
+  tiles[f].type = type_info[t].type;
+  tiles[f].image = type_info[t].image[idx] ;
 
   assert (dep_tree[f].free) ;
   assert (!dep_tree[f].filled) ;
-  dep_tree[f].filled = 1 ; // Lord
+  dep_tree[f].filled = 1 ; /* Lord */
   dep_tree[f].free = 0 ;
 
-  /* Now let the tiles near this one see if this makes them 'free' */
-  /* N.B. You shall know the truth and the truth will set you free */
+  /* Now let the tiles near this one see if this makes them 'free'
+   * N.B. You shall know the truth and the truth will set you free */
   {
     int lp, i ;
     for (lp=0;lp<4;lp++)
@@ -321,26 +403,10 @@ int random_tile_type ()
   int o ;
   do
     o = random_UDL_half ;
-  while (sensible_types[o].placed) ;
+  while (type_info[o].placed) ;
   return o ;
 }
 
-/* This finds the offset into default_pos of a tile given its x,y and layer
-   it returns -1 if there is no tile at that position.
-   N.B. Each tile occupies 2x2 positions.
- */
-int tile_at (int layer, int x, int y)
-{
-  int lp ;
-
-  for (lp=0;lp<MAX_TILES;lp++)
-    if ((default_pos[lp].x==x || default_pos[lp].x==x+1) &&
-	(default_pos[lp].y==y || default_pos[lp].y==y+1) &&
-	(default_pos[lp].layer == layer))
-      return lp ;
-  return -1 ;
-}
-
 /* Remove duplicate entries */
 void unique (int *a, int l)
 {
@@ -365,32 +431,44 @@ void unique (int *a, int l)
 }
 
 /* Generate a single tiles dependancy tree entries */
-/* N.B. Layer increases as you go up */
+/* N.B. Layer increases as you go up tiles BRC is x,y x+right y+down*/
 void tile_deps (int tn)
 {
-  int lp, layer, x, y, lpx, lpy ;
+  int lp, layer, x, y, lpx, lpy, d ;
 
-  // Side dependancies
-  x = default_pos[tn].x ;
-  y = default_pos[tn].y ;
-  layer = default_pos[tn].layer ;
-  
-  dep_tree[tn].lhs_dep[0] = tile_at(layer, x-2, y) ;
-  dep_tree[tn].lhs_dep[1] = tile_at(layer, x-2, y-1) ;
+  /* Side dependancies */
+  x = pos[tn].x ;
+  y = pos[tn].y ;
+  layer = pos[tn].layer ;
 
-  dep_tree[tn].rhs_dep[0] = tile_at(layer, x+2, y) ;
-  dep_tree[tn].rhs_dep[1] = tile_at(layer, x+2, y-1) ;
-  
-  // Place / take dependancies
+  d = dep_tree[tn].lhs_dep[0] = tile_at(layer, x-2, y) ;
+  d = dep_tree[tn].lhs_dep[1] = tile_at(layer, x-2, y-1) ;
+
+  d = dep_tree[tn].rhs_dep[0] = tile_at(layer, x+1, y) ;
+  d = dep_tree[tn].rhs_dep[1] = tile_at(layer, x+1, y-1) ;
+
+  /* Place / take dependancies */
   lp=0 ;
-  for (lpx=0;lpx<=1;lpx++)	
-    for (lpy=0;lpy<=1;lpy++)
+  for (lpx=0;lpx<=1;lpx++)	/* RHS -> LHS */
+    for (lpy=0;lpy<=1;lpy++)	/* BOT -> TOP */
       {
 	dep_tree[tn].turn_dep[lp] = tile_at(layer+1, x-lpx, y-lpy) ;
 	dep_tree[tn].place_dep[lp] = tile_at(layer-1, x-lpx, y-lpy) ;
 	lp++ ;
       }
-  dep_tree[tn].filled = 0 ; // Lord
+#ifdef PLACE_DEBUG
+  printf ("Generated :\n") ;
+  dump_deps(tn) ;
+  /* Nastyness treat structure as int* union */
+  {
+  int *ptr = dep_tree[tn].turn_dep ;
+  int lp ;
+  for (lp=0;lp<12;lp++)
+      assert (*ptr++!=tn) ;
+  }
+#endif
+
+  dep_tree[tn].filled = 0 ; /* Lord */
   dep_tree[tn].free = 0 ;
 
   unique(dep_tree[tn].turn_dep, 4) ;
@@ -403,7 +481,7 @@ void tile_deps (int tn)
    to determine which tiles can be turned / how to place the tiles */
 void generate_dependancies ()
 {
-  int lp, num, tile ;
+  int lp ;
 
   for (lp=0;lp<MAX_TILES;lp++)
     {
@@ -412,29 +490,14 @@ void generate_dependancies ()
       dump_deps(lp) ;
 #endif
     }
-
-  /* Build a nice list of pairs of tiles */
-  tile = 0 ;
-  num = 0 ;
-  for (lp=0;lp<MAX_TILES;lp++)
-    {
-      sensible_types[lp/2].def[(lp&1)] = tile ;
-      num++;
-      if (num>=default_types[tile].num_tiles)
-	{
-	  num = 0 ;
-	  tile++ ;
-	}
-    }
 }
 
 /* Do the tile placement for a soluable game */
 void generate_game (void)
 {
-  static int fails = 0 ;
   static int generation = 0 ;
   int i, lp;
-  
+
   /* If this bites please mail me as above */
   for (lp=0;lp<MAX_TILES*MAX_TILES;lp++)
     {
@@ -443,20 +506,26 @@ void generate_game (void)
       assert (i<MAX_TILES) ;
     }
   
-  if ((i=setjmp(unsolvable))==1)
+  if ((i=setjmp(unsolvable))!=0)
     {
       fails++ ;
-      printf ("Warning - impossible seed\n") ;
-      printf ("%d fails in %d generations\n", fails, generation) ;
+      if (fails%10==0)
+        {
+	  printf ("Warning - impossible seed %d\n", i) ;
+	  printf ("%d fails in %d generations\n", fails, generation) ;
+#ifdef PLACE_DEBUG
+          keyPress() ;
+#endif
+        }
     }
   generation++ ;
   
   for (lp = 0; lp < MAX_TILES; lp++)
     {
       tiles[lp].visible = 0;
-      dep_tree[lp].free = 0 ;
-      dep_tree[lp].filled = 0 ; // Lord
-      sensible_types[lp/2].placed = 0 ;
+      dep_tree[lp].free = 0 ; 
+      dep_tree[lp].filled = 0 ; /* Lord */
+      type_info[lp/2].placed = 0 ;
     }
   
   /* OK ! we want only one tile per apparent horizontal line ! */
@@ -466,6 +535,13 @@ void generate_game (void)
       if (ok_free_validate_line (t))
 	dep_tree[t].free = 1 ;
     }
+  /* Check we really got them all ? : very important for algorithem ! */
+  for (lp=0;lp<MAX_TILES;lp++)
+    {
+	if (tiles[lp].layer==0 && 
+	    ok_free_validate_line (lp))
+	    dep_tree[lp].free = 1 ;
+    }
 #ifdef PLACE_DEBUG
   printf ("Done\n") ;
   
@@ -476,47 +552,53 @@ void generate_game (void)
 #endif
   
   {
-    int last_place, tileo, tile, offset;
+    int last_place, tile, offset, offset2;
 
-    last_place = tileo = tile = offset = -1;
+    last_place = tile = offset = -1;
     
     /* Insert */
     for (lp=0;lp<MAX_TILES;lp++)
       {
-	/* Find somewhere to put it */
-	if ((lp&0x1)==0)	// Every other tile depends on the last
+	/* Find somewhere to put it: every other tile depends on the last */
+	if ((lp&0x1)==0)
 	  {
-	    tileo = random_tile_type() ;
-	    sensible_types[tileo].placed = 1 ;
+	    tile = random_tile_type() ;
+	    type_info[tile].placed = 1 ;
 	    last_place = -1 ;
 #ifdef PLACE_DEBUG
-	    if (global_wait)
-	      {
-		int zlp, b ;
-		clear_window() ;
-		for (zlp=0;zlp<MAX_TILES;zlp++)
-		  redraw_tile(zlp) ;
-		for (zlp=0;zlp<MAX_TILES;zlp++)
-		  tiles[zlp].selected = 0 ;
-		while (fread(&b, 1, 1, stdin)==0) ;
-	      }
+	    keyPress() ;
 #endif
 	  }
-	tile = sensible_types[tileo].def[(lp&1)] ;
+
 	offset = random_free (last_place) ;
-	place_tile (offset, tile) ;
+	if (lp<MAX_TILES-1) /* There is another tile */
+	  {
+	    offset2 = random_free (last_place) ;
+	    if (pos[offset2].layer > pos[offset].layer)
+	      offset = offset2 ;  /* Higher = better... */
+	  }
+	place_tile (offset, tile, (lp&1)) ;
 	
 #ifdef PLACE_DEBUG
-	printf ("lp %d, lp&01 %d tileo %d, tile %d\n", lp, (lp&0x1), tileo, tile) ;
+	printf ("lp %d, lp&01 %d, tile %d\n", lp, (lp&0x1), tile) ;
 	printf ("Place %d at : %d\n", tile, offset) ;
 	tiles[offset].selected = 1 ;
 #endif
-	
+#ifdef CHEAT_DEBUG
+#warning Cheater
 	tiles[offset].sequence = (MAX_TILES/2) - (int)(lp/2) ;
+#else
+	tiles[offset].sequence = 0 ;
+#endif
 	last_place = offset ;
       }
   }
 #ifdef PLACE_DEBUG
+  {
+      int lp ;
+      for (lp=0;lp<MAX_TILES;lp++)
+	  tiles[lp].selected = 0 ;
+  }
   printf ("Finished : Draw !\n") ;
   global_wait = 1 ;
 #endif
